function [AC] = FASER_config()
% function [AC] = FASER_config()
%
%
% FASER configuration file. Sets aircraft parameters.
% Called from: UAV_config.m
%
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
%
% SVN Info: $Id: FASER_config.m 284 2011-03-03 15:07:19Z murch $


AC.aircraft = 'FASER';

%% Inertia, Mass and CG location
% Inertia data taken from NASA FASER_SIM_NOAH faser_massprop.m

% Gross aircraft mass [kg] (including relevant hardware)
AC.Mass  = 6.804; % kgs (15 lbs)

% CG location [x y z], [m]
AC.Geometry.rCG = [0.2175 0 0]; %CG set to be Quarter-chord point

% Gross moments of inertia [Jx Jy Jz Jxz] [kg*m^2]
AC.Inertia.Ixx = 0.672485702;
AC.Inertia.Iyy = 0.889416574;
AC.Inertia.Izz = 1.57817209;
AC.Inertia.Ixz = -0.759258051;
AC.Inertia.Matrix = [AC.Inertia.Ixx  0               -AC.Inertia.Ixz;...
    0         AC.Inertia.Iyy  0;...
    -AC.Inertia.Ixz 0               AC.Inertia.Izz];


%% Aircraft Geometric Parameters
% Aerodynamic force application point (usually the aerodynamic center)[x y z]
AC.Geometry.AeroCenter = [0.2175 0 0];

% Geometry data from NASA FASER_SIM_NOAH faser_deq.m
% Mean aerodynamic chord [m]
AC.Geometry.c = 0.433578; % 0.4279 m (calculated MAC)
% Wing span [m]
AC.Geometry.b = 1.917192; % 1.92 m aft max
% Wing area [m^2]
AC.Geometry.S = 0.769237171; %.7732 from measurements

%Control scaling factor
% scale_control = 0.4363;
AC.Geometry.scale_control = 1;

% Remarks:
% ALL stability derivative units are radians:

%% Aero coefficients
AC.Aero = load('FASER_Aero');
AC.Aero.use_FASER_aero = 1;

%% PROPULSION
%Propulsion force application point (usually propeller hub) [x y z]

AC.Geometry.rProp = [-0.075 0 0]; % [m] Propeller in front of reference point

% Using an APC 12 x 6E propeller, EFlite Power 25 motor
% Coefficient of thrust polynomial model 
% AEM wind tunnel test data Jan 2011
AC.Prop.CT = [-0.4314 1.08 -0.896 0.1089 0.0604 ];

% Coefficient of power polynomial model 
% AEM wind tunnel test data Jan 2011
AC.Prop.CP = [0.5054 -0.5304 0.0412 0.01664 0.0223];

% Motor polynomial model. Input throttle, output power in watts
% EFlite Power 25, AEM wind tunnel test data Jan 2011
% Castle Creations ICE Lite 50 used to collect motor data
AC.Prop.Power = [174.46 70.135 -4.39]*6;

% Electric motor and propeller combine moment of inertia
AC.Prop.Jmp = 0.00012991;  % [kg*m^2] From experimental measurement (not updated with Jan 2011 data)

% Propeller diameter (m) (14x9.5 prop on FASER)
AC.Prop.Radius = 0.1778;

AC.Prop.ThrottleOutputLimit.Upper = 1900;
AC.Prop.ThrottleOutputLimit.Lower = 0;
AC.Prop.OmegaSaturation.Upper = 15000*pi/30;
AC.Prop.OmegaSaturation.Lower = 1;


% Electric motor and propeller combine moment of inertia
AC.Prop.Jmp = 0.00012991;  % [kg*m^2] From experimental measurement

%% Configure Actuators and Initial Conditions

d2r = pi/180;                         % Degrees to Radians conversion

%Elevator
AC.Actuator.elevator.BW = 8;    % [Hz]
AC.Actuator.elevator.RateLim = 150*d2r;   % [rad/s]
AC.Actuator.elevator.PosLim = 25*d2r;  % [rad]
AC.Actuator.elevator.NegLim = -25*d2r; %[rad]

%Aileron
AC.Actuator.aileron.BW = 8;    % [Hz]
AC.Actuator.aileron.RateLim = 150*d2r;   % [rad/s]
AC.Actuator.aileron.PosLim = 25*d2r;  % [rad]
AC.Actuator.aileron.NegLim = -25*d2r; %[rad]

%Rudder
AC.Actuator.rudder.BW = 8;    % [Hz]
AC.Actuator.rudder.RateLim = 150*d2r;   % [rad/s]
AC.Actuator.rudder.PosLim = 25*d2r;  % [rad]
AC.Actuator.rudder.NegLim = -25*d2r; %[rad]

%Flap
AC.Actuator.flap.BW = 8;    % [Hz]
AC.Actuator.flap.RateLim = 150*d2r;   % [rad/s]
AC.Actuator.flap.PosLim = 25*d2r;  % [rad]
AC.Actuator.flap.NegLim = -25*d2r; %[rad]

%Throttle
AC.Actuator.throttle.PosLim = 1;  % [nd]
AC.Actuator.throttle.NegLim = 0; %[nd]

%% Configure Sensor Noise Parameters
AC.Sensors.NoiseOn = 1;
AC.Sensors.IMU.p_noise = 0.000001; % rad/s
AC.Sensors.IMU.q_noise = 0.000001; % rad/s
AC.Sensors.IMU.r_noise = 0.000001; % rad/s
AC.Sensors.IMU.hx_noise =150;   % nT 
AC.Sensors.IMU.hy_noise =150;   % nT
AC.Sensors.IMU.hz_noise =80000; % nT
AC.Sensors.IMU.ax_noise =0.0008; % m/s^2
AC.Sensors.IMU.ay_noise =0.004;  % m/s^2
AC.Sensors.IMU.az_noise =0.004;  % m/s^2
AC.Sensors.AirData.ias_noise = 0.001; % m/s
AC.Sensors.AirData.h_noise = 0.02; % m
