function [longmod,latmod,linmodel,TF]=linearize_UAV(OperatingPoint,verbose)
% [longmod,latmod,linmodel,TF]=linearize_UAV(OperatingPoint,verbose)
%
% Linearizes the UAV model about a given operating point using
% ../NL_Sim/UAV_NL.mdl. This function can be called from any of the three
% sim directories. However, this function will use your workspace
% variables. Requires the Control System Toolbox and Simulink Control
% Design.
%
% Inputs:
%   OperatingPoint - Operating point object of a trim condition
%   verbose        - boolean flag to suppress output; default "true"
%
% Outputs:
%   longmod  - longitudinal linear model
%   latmod   - lateral directional linear model
%   linmodel - full linear model
%   TF       - common input/output transfer functions
%
% Calls: eigpara.m
%
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
%
% SVN Info: $Id: linearize_UAV.m 284 2011-03-03 15:07:19Z murch $

% verbose output on by default
if nargin < 2
    verbose = 1;
end

%% Load model into memory
isLoaded = bdIsLoaded('UAV_NL'); % check to see if model is loaded or open
if ~isLoaded
    load_system('../NL_Sim/UAV_NL.mdl') % load model into system memory without opening diagram
end

%% LINEARIZE
linmodel = linearize('UAV_NL',OperatingPoint.op_point);
% linC = [linmodel.C(1:7,:); zeros(3,4) eye(3) zeros(3,6)];
% linD = [zeros(7,4); zeros(3,4)];
% 
% linmodel = ss(linmodel.A,linmodel.B,linC,linD);  clear linC linD

% Create Full 13-state Linear Model
set(linmodel, 'InputName', {'\delta_e' ;'\delta_a'; '\delta_r';'\delta_t';'\delta_f'});
set(linmodel, 'OutputName',{'V'; '\beta'; '\alpha'; 'h'; '\phi'; '\theta';'\psi';'p';'q';'r';'gamma'});
set(linmodel, 'StateName', {'\phi';'\theta';'\psi';'p';'q';'r';'u';'v';'w';'Xe';'Ye';'Ze';'\omega'})

% Generate transfer functions from statespace data
[num,den] = ss2tf(linmodel.A,linmodel.B,linmodel.C,linmodel.D,1);  % TFs from elev to outputs
% elev to output 9 (q)
TF.full_elev2q = tf(num(9,:),den,'InputName',{'\delta_e'},'OutputName',{'q'});

[num,den] = ss2tf(linmodel.A,linmodel.B,linmodel.C,linmodel.D,2);  % TFs from ail to outputs
% ail to output 8 (p)
TF.full_ail2p = tf(num(8,:),den,'InputName',{'\delta_a'},'OutputName',{'p'});
% ail to output 10 (r)
TF.full_ail2r = tf(num(10,:),den,'InputName',{'\delta_a'},'OutputName',{'r'});

[num,den] = ss2tf(linmodel.A,linmodel.B,linmodel.C,linmodel.D,3);  % TFs from rud to outputs
% rud to output 8 (p)
TF.full_rud2p = tf(num(8,:),den,'InputName',{'\delta_r'},'OutputName',{'p'});
% rud to output 10 (r)
TF.full_rud2r = tf(num(10,:),den,'InputName',{'\delta_r'},'OutputName',{'r'});


%% GENERATE LONGITUDINAL LINEAR MODEL
% Longitudinal dynamics
% States: u(#7)  w(#9)  q(#5) theta(#2) Ze(#12) omega(#13)
% Inputs: elevator(#1) throttle(#4)
% Outputs: Vs(#1) alpha(#3) q(from states) theta(from states) h (#7)

% Generate State-space matrices for Longitudinal Model
stateind = [7 9 5 2 12 13];
A_lon = [
    linmodel.A(7, stateind)
    linmodel.A(9, stateind)
    linmodel.A(5 ,stateind)
    linmodel.A(2 ,stateind)
    linmodel.A(12,stateind)    
    linmodel.A(13,stateind)
    ];

B_lon = [
    linmodel.B(7, [1 4])
    linmodel.B(9, [1 4])
    linmodel.B(5, [1  4])
    linmodel.B(2, [1 4])
    linmodel.B(12, [1 4])    
    linmodel.B(13, [1 4])
    ];
C_lon = [
    linmodel.C(1,stateind)
    linmodel.C(3,stateind)
    zeros(2, 6)
    linmodel.C(4,stateind)
    ];
C_lon(3,3) = 1; 
C_lon(4,4) = 1;

D_lon = zeros(5,2);

% Combine into statespace data type named 'longmod'
longmod = ss(A_lon,B_lon, C_lon, D_lon); clear A_lon B_lon C_lon D_lon
% Label system inputs, outputs, and states
set(longmod, 'InputName', {'\delta_e [rad]'; '\delta_t [%/100]'});
set(longmod, 'OutputName',{'V_s [m/s]'; '\alpha [rad]'; 'q [rad/s]'; '\theta [rad]'; 'h [m]'})
set(longmod, 'StateName', {'u [m/s]';'w [m/s]';'q [rad/s]';'\theta [rad]';'Ze [m]';'\omega [rad/s]'})

% Generate transfer functions from statespace data
[num,den] = ss2tf(longmod.A,longmod.B,longmod.C,longmod.D,1);  % TFs from elev to outputs

% elev to output 1 (V)
TF.elev2Vs = tf(num(1,:),den,'InputName',{'\delta_e [rad]'},'OutputName',{'V_s [m/s]'});
% elev to output 2 (alpha)
TF.elev2alpha = tf(num(2,:),den,'InputName',{'\delta_e [rad]'},'OutputName',{'\alpha [rad]'});
% elev to output 3 (q)
TF.elev2q = tf(num(3,:),den,'InputName',{'\delta_e [rad]'},'OutputName',{'q [rad/s]'});
% elev to output 4 (theta)
TF.elev2theta = tf(num(4,:),den,'InputName',{'\delta_e [rad]'},'OutputName',{'\theta [rad]'});
% elev to output 5 (h)
TF.elev2h = tf(num(5,:),den,'InputName',{'\delta_e [rad]'},'OutputName',{'h [m]'});

[num,den] = ss2tf(longmod.A,longmod.B,longmod.C,longmod.D,2);  % TFs from thr to outputs
% thr to output 1 (V)
TF.thr2Vs = tf(num(1,:),den,'InputName',{'\delta_t [%/100]'},'OutputName',{'V_s [m/s]'});
% thr to output 2 (alpha)
TF.thr2alpha = tf(num(2,:),den,'InputName',{'\delta_t [%/100]'},'OutputName',{'\alpha [rad]'});
% thr to output 3 (q)
TF.thr2q = tf(num(3,:),den,'InputName',{'\delta_t [%/100]'},'OutputName',{'q [rad/s]'});
% thr to output 4 (theta)
TF.thr2theta = tf(num(4,:),den,'InputName',{'\delta_t [%/100]'},'OutputName',{'\theta [rad]'});
% thr to output 5 (h)
TF.thr2h = tf(num(5,:),den,'InputName',{'\delta_t [%/100] '},'OutputName',{'h [m]'});

if verbose
    fprintf('\n');
    fprintf('\n Longitudinal Dynamics');
    fprintf('\n-----------------------');
    fprintf('\n  State vector: x = [u w q theta Ze Omega]');
    fprintf('\n  Input vector: u = [elevator throttle]');
    fprintf('\n Output vector: y = [Vs alpha q theta h]');
    fprintf('\n State matrix: A = \n\n\n');
    disp(longmod.A);
    fprintf('Longitudinal Poles:');
    damp(longmod.A)
    fprintf('\n\n\n Control matrix: B = \n');
    disp(longmod.B);
    fprintf('\n Observation matrix: C = \n');
    disp(longmod.C);
    
    % Eigenvalue analysis
    eiglon = eig(longmod.A);
    for i=1:length(eiglon);
        if imag(eiglon(i))>0
            [wd, T, wn, zeta] = eigpara(eiglon(i));
            fprintf('\n Eigenvalue: %6.4f +/- %6.4f i', real(eiglon(i)), imag(eiglon(i)));
            fprintf('\n Damping = %6.4f, natural frequency = %6.4f rad/s, period = %8.4f s', zeta, wn, T);
        elseif imag(eiglon(i))==0
            fprintf('\n Eigenvalue: %6.4f', eiglon(i));
            fprintf('\n Time constant = %6.4f s', -1/eiglon(i));
        end
    end
end
%% GENERATE LATERAL-DIRECTIONAL LINEAR MODEL
% Lateral-directional dynamics
% States: v(#8) p(#4) r(#6) phi(#1) psi(#3)
% Inputs: aileron(#2) rudder(#3)
% Outputs: beta(#2) p(from states) r(from states) phi(from states) psi(from states)
stateind = [8 4 6 1 3];
A_lat = [
    linmodel.A(8,stateind)
    linmodel.A(4,stateind)
    linmodel.A(6,stateind)
    linmodel.A(1,stateind)
    linmodel.A(3,stateind)
    ];

B_lat = [
    linmodel.B(8, [2 3])
    linmodel.B(4, [2 3])
    linmodel.B(6, [2 3])
    linmodel.B(1, [2 3])
    linmodel.B(3, [2 3])
    ];
C_lat = [
    linmodel.C(2,stateind)
    zeros(4, 5)
    ];
C_lat(2,2) = 1; C_lat(3,3) = 1;
C_lat(4,4) = 1; C_lat(5,5) = 1;

D_lat = zeros(5,2);

% Combine into statespace data type named 'latmod'
latmod = ss(A_lat,B_lat, C_lat, D_lat);
% Label system inputs, outputs, and states
set(latmod, 'InputName', {'delta_a [rad]'; 'delta_r [rad]'});
set(latmod, 'OutputName',{'beta [rad]'; 'p [rad/s]'; 'r [rad/s]'; 'phi [rad]'; 'psi [rad]'})
set(latmod, 'StateName', {'v [m/s]';'p [rad/s]';'r [rad/s]';'phi [rad]';'psi [rad]'})

% Generate transfer functions from statespace data
[num,den] = ss2tf(latmod.A,latmod.B,latmod.C,latmod.D,1);  % TFs from ail to outputs

% ail to output 1 (v)
TF.ail2v = tf(num(1,:),den,'InputName',{'\delta_a [rad]'},'OutputName',{'v [m/s]'});
% ail to output 2 (p)
TF.ail2p = tf(num(2,:),den,'InputName',{'\delta_a [rad]'},'OutputName',{'p [rad/s]'});
% ail to output 3 (r)
TF.ail2r = tf(num(3,:),den,'InputName',{'\delta_a [rad]'},'OutputName',{'r [rad/s]'});
% ail to output 4 (phi)
TF.ail2phi = tf(num(4,:),den,'InputName',{'\delta_a [rad]'},'OutputName',{'\phi [rad]'});
% ail to output 5 (psi)
TF.ail2psi = tf(num(5,:),den,'InputName',{'\delta_a [rad]'},'OutputName',{'\psi [rad]'});

[num,den] = ss2tf(latmod.A,latmod.B,latmod.C,latmod.D,2);  % TFs from rud to outputs

% rud to output 1 (v)
TF.rud2v = tf(num(1,:),den,'InputName',{'\delta_r [rad]'},'OutputName',{'v [m/s]'});
% rud to output 2 (p)
TF.rud2p = tf(num(2,:),den,'InputName',{'\delta_r [rad]'},'OutputName',{'p [rad/s]'});
% rud to output 3 (r)
TF.rud2r = tf(num(3,:),den,'InputName',{'\delta_r [rad]'},'OutputName',{'r [rad/s]'});
% rud to output 4 (phi)
TF.rud2phi = tf(num(4,:),den,'InputName',{'\delta_r [rad]'},'OutputName',{'\phi [rad]'});
% rud to output 5 (psi)
TF.rud2psi = tf(num(5,:),den,'InputName',{'\delta_r [rad]'},'OutputName',{'\psi [rad]'});

% Variable cleanup
clear num den A_lat B_lat C_lat D_lat

if verbose
    fprintf('\n');
    fprintf('\n Lateral-directional Dynamics');
    fprintf('\n------------------------------');
    fprintf('\n  State vector: x = [v p r phi psi]');
    fprintf('\n  Input vector: u = [aileron rudder]');
    fprintf('\n Output vector: y = [beta p r phi psi]');
    fprintf('\n State matrix: A = \n\n\n');
    disp(latmod.A);
    fprintf('Lateral-directional Poles:');
    damp(latmod.A)
    fprintf('\n\n\n Control matrix: B = \n');
    disp(latmod.B);
    fprintf('\n Observation matrix: C = \n');
    disp(latmod.C);
    
    % Eigenvalue analysis
    eiglat = eig(latmod.A);
    for i=1:length(eiglat)
        if imag(eiglat(i))>0
            [wd, T, wn, zeta] = eigpara(eiglat(i));
            fprintf('\n Eigenvalue: %6.4f +/- %6.4f i', real(eiglat(i)), imag(eiglat(i)));
            fprintf('\n Damping = %6.4f, natural frequency = %6.4f rad/s, period = %8.4f s', zeta, wn, T);
        elseif (imag(eiglat(i))==0)&&(real(eiglat(i))~=0)
            fprintf('\n Eigenvalue: %6.4f', eiglat(i));
            fprintf('\n Time constant = %6.4f s', -1/eiglat(i));
        end
    end
    fprintf('\n')
    clear wd T wn zeta eiglon eiglat i
end

%% Cleanup
if ~isLoaded
    bdclose UAV_NL % clear model from system memory if we had to load it
end