/******************************************************************************
 * FILE: L_control_doublet.c
 *
 * DESCRIPTION:
 * Inner loop (pitch and roll command tracking controllers) for Ultrastick
 * research AV. Preprogrammed doublet commands test functionality provided.
 *
 * ORIGINAL AUTHOR: Rohit Pandita
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: control_manager.c 253 2011-01-20 22:01:35Z murch $
 ******************************************************************************/

#include <stdio.h>
#include <cyg/posix/time.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <cyg/io/mpc5xxx_gpio.h>

#include "control_law_interface.h"

//#include "control_cas.h"
#include "control_defs.h"

#define 	half_pi			1.5707963267948966192313216916397514

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//global variables
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

extern short whichmode[];

extern void endian_swap( unsigned char *buf, int index, int count );
double doublet (double t0, double currentTime, double duration, double amplitude);

float  normalizedControl (float pos, float maxControl);

static double t0;
word    delr,dele,dela,delthr;
word    delf_l,delf_r,dela_l,dela_r;
static   double  p_corr, q_corr, r_corr;
static   double  h_f=0,ias_f=0,r_f=0;
static   double  filtered_alt, filtered_ias, filtered_yawrate, tnow_c, dt;
static   double  h_f_p=0, ias_f_p=0, r_f_p=0, tprev_c=0;

static struct imu      imuval;
static struct gps      gpsval;
static struct nav      navval;

static   double  pitch_dmd, roll_dmd;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//control code
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void *control_uav(void *thread_id)
{

	double references[2]; // Control references
	double *commands; // Actuator deflections
	double feedback[17]; // Feedback signals


	static int t0_latched=FALSE;
	int rxmux_PersistanceCheck=0;
	//unsigned int width_ch6;
	unsigned int rxmux_status;
	float base_pitch_dmd= 5*D2R;

	int 	rc;
	double  toc;
	static double tic;
	
	// Variables for control commands
	double  da=0, de=0, dthr=0, dr=0, da_l=0, da_r=0, df_l=0, df_r=0;

	// variables for last control command
	static double de_last = 0;
	static double da_last = 0;
	static double dr_last = 0;

	sprintf(&msg[3],"CONTROL thread created");
	write(sPort0, msg, sizeof(msg));


	while( ofpMode != terminate )
	{
		pthread_mutex_lock(&mutex_imu);
		rc = pthread_cond_wait (&trigger_claw, &mutex_imu);

		if (rc == 0) {
			// compute thread jitter
			toc = get_Time();
			threadstat[3].jitter_nsec = ( (toc-tic) - 1/(double)CLAW_Hz ) * 1E09;
			tic = toc;

			pthread_mutex_lock(&mutex_fdat);
			pthread_mutex_lock(&mutex_gps);
			pthread_mutex_lock(&mutex_nav);
			imuval = imupacket;                    // save the last attitude
			gpsval = gpspacket;                    // save the last gps
			navval = navpacket;                    // save the last nav
			pthread_mutex_unlock(&mutex_nav);
			pthread_mutex_unlock(&mutex_gps);

			// apply estimated gyro bias correction
			p_corr = imuval.p - xs[4];
			q_corr = imuval.q - xs[5];
			r_corr = imuval.r - xs[6];

			tnow_c = get_Time();
			dt = tnow_c - tprev_c;
			if (dt == 0.0) dt = cdT;
			tprev_c = tnow_c;

			//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			//pass through the first order low pass filter to remove noise
			//G(s)=1/(tau s + 1), time constant =0.4;
			//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			h_f   = 0.9048*h_f_p + 0.09516*(imuval.h);
			h_f_p = h_f;
			filtered_alt = h_f;

			ias_f   = 0.9048*ias_f_p + 0.09516*(imuval.ias);
			ias_f_p = ias_f;
			filtered_ias = ias_f;

			r_f   = 0.8187*r_f_p + 0.1813*(r_corr);		// time constant=0.4
			r_f_p = r_f;
			filtered_yawrate = r_f;

			// RxMux states: RC=low, MPC=high(3.3V)
			rxmux_status = GPIO_GPW_GetInputStatus(MPC5XXX_GPW_GPIO_WKUP_6);

			if ( (rxmux_status == TRUE) && (rxmux_PersistanceCheck++ > 10) )
			{
				rxmux_PersistanceCheck--;

				if(t0_latched == FALSE) {
					t0=get_Time();
					t0_latched = TRUE;
					// set OFP mode status flag
					whichmode[0] = TRUE;
				}
				
				// inner loop control (attitude tracking)

				// Assign doublet sequences to references vector
				references[0] = doublet(t0+2, get_Time(), 6, 20*D2R); // Roll angle command
				references[1] = doublet(t0+8, get_Time(), 6, 5*D2R) + base_pitch_dmd; // Pitch angle command

				roll_dmd = references[0];
				pitch_dmd = references[1];

				// Baseline controller
				//de = pitch_claw (base_pitch_dmd+pitch_dmd, imuval.the, q_corr, imuval.phi, dt);
				//da = roll_claw (roll_dmd, imuval.phi, p_corr, dt);
				//dr = yawdamper (r_corr);

				// LQR controller
				
				//pitch_claw_lqr (base_pitch_dmd+pitch_dmd, imuval.the, q_corr, dt, &de );
				//roll_claw_lqr (roll_dmd, imuval.phi, p_corr, r_corr, dt, &da, &dr);

				// Need: Set sample time to control
				
				//create feedback vector
				feedback[0] = imuval.phi;
				feedback[1] = imuval.the;
				feedback[2] = imuval.psi;
				feedback[3] = p_corr;
				feedback[4] = q_corr;
				feedback[5] = r_corr;
				feedback[6] = imuval.ax;
				feedback[7] = imuval.ay;
				feedback[8] = imuval.az;
				feedback[9] = imuval.h;
				feedback[10] = imuval.ias;
				feedback[11] = gpsval.lat;
				feedback[12] = gpsval.lon;
				feedback[13] = gpsval.alt;
				feedback[14] = gpsval.vn;
				feedback[15] = gpsval.ve;
				feedback[16] = gpsval.vd;

				commands = get_control_outputs(feedback, references, get_Time(),dt);

				da = commands[0]; // aileron
				de = commands[1]; // elevator
				dr = commands[2]; // rudder


				// Assign commands to left/right aileron and flaps, and throttle
				da_l=-da;
				da_r=da;
				df_l = df_r = 0;
				dthr = 0.7;

								
			}
			else if ( (rxmux_status == FALSE) && (rxmux_PersistanceCheck-- <= 0) )
			{
				t0_latched = FALSE;
				whichmode[0] = FALSE;
				rxmux_PersistanceCheck=0;

				de=0.075; // rad, approximate trim value
				da=0;
				da_l = da_r = 0;
				df_l = df_r = 0;
				dr=0;
				dthr=0.7;
				//integrator[0] = integrator[1] = 0; // reset integrator state to zero
				//anti_windup[0] = anti_windup[1] = 1;	// reset integrator anti_windup
				reset_control();
				
			}

			controlpacket.da = da;
			controlpacket.de = de;
			controlpacket.dr = dr;
			controlpacket.dthr = dthr;

			// Convert control surface commands to PWM
			delr   = (word)(PWM_1ms_COUNTS + PWM_COUNTS_RANGE*normalizedControl(-dr,RUD_YAWDAMP_AUTH_MAX));
			dela   = (word)(PWM_1ms_COUNTS + PWM_COUNTS_RANGE*normalizedControl(-da,AILERON_MAX));
			dele   = (word)(PWM_1ms_COUNTS + PWM_COUNTS_RANGE*normalizedControl(de,ELEVATOR_MAX));
			delthr = (word)(PWM_1ms_COUNTS + PWM_COUNTS_RANGE*dthr);

			dela_l = dela_r = dela; // aileron servos are physically reversed, so same command produces opposite deflections
			delf_l = delf_r = (word)( PWM_1ms_COUNTS + PWM_COUNTS_RANGE/2 );	// no cal for flaps at this time			
			
			// Write servo commands to mpc5200 PWM channels
			GPT_PWM_Write_Width( 0, delf_l);		// left flap
			GPT_PWM_Write_Width( 1, delf_r);		// right flap
			GPT_PWM_Write_Width( 2, dela_r);		// right aileron (aileron Y'd)
			GPT_PWM_Write_Width( 7, dela_l);		// left aileron	
			GPT_PWM_Write_Width( 3, dele);          // elevator
			GPT_PWM_Write_Width( 4, delthr);        // throttle
			GPT_PWM_Write_Width( 5, delr);			// rudder	
			
			// Store data
			fltdatpacket.pitch_ref = (float)(pitch_dmd+base_pitch_dmd);
			fltdatpacket.roll_ref = (float)roll_dmd;
			fltdatpacket.de = (float)de;
			fltdatpacket.da_l = (float)da_l;  // left aileron
			fltdatpacket.da_r = (float)da_r;  // right aileron
			fltdatpacket.df_l = (float)df_l;  // left flap
			fltdatpacket.df_r = (float)df_r;  // right flap
			fltdatpacket.dr = (float)dr;
			fltdatpacket.dthr = (float)dthr;
			fltdatpacket.mode = (short)whichmode[0];

#ifdef DATALOG_LSB
			endian_swap( (byte *)&(fltdatpacket.pitch_ref), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.roll_ref), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.de), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.da_l), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.da_r), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.df_l), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.df_r), 0, 4 );			
			endian_swap( (byte *)&(fltdatpacket.dr), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.dthr), 0, 4 );
			endian_swap( (byte *)&(fltdatpacket.mode), 0, sizeof(fltdatpacket.mode) );
#endif
			// release mutexes
			pthread_mutex_unlock(&mutex_fdat);
			pthread_mutex_unlock(&mutex_imu);
		}

	}

	sprintf(&msg[3],"CONTROL thread terminated.");
	write(sPort0, msg, sizeof(msg));
	pthread_exit(NULL);
}

// +++++++++++++++++++++++++++++++++++ functions ++++++++++++++++++++++++++++++++++++++++
double satfun(double dta,double max)
{
	double temp=0;

	if (fabs(dta) > max)
		temp = sign(dta)*max;
	else
		temp = dta;

	return temp;
}

/* function 'normalizedControl' generates control signal normailzed to the range 0 to 1*/
float normalizedControl (float pos, float maxControl)
{
	float normControl;

	pos = satfun (pos, maxControl);
	normControl = (maxControl + pos)/(2.0*maxControl);

	return(normControl);
}


double doublet (double t0, double currentTime, double duration, double amplitude)
{
	double t = currentTime;	// simulink current time

	if ( t < t0 )
		return 0;
	else if ( (t >= t0) && (t <= t0 + 0.5*duration) )
		return amplitude;
	else if ( (t >= t0 + 0.5*duration) && (t <= t0 + duration) )
		return -amplitude;
	if ( t > t0 + duration )
		return 0;

	return 0;
}
/*****************************************************************************/



